@@ -49,10 +49,24 @@ class TestCase():
         return self._data.get(arg, default)
 
 
-SPEC_SCHEMA = Schema(
+TYPE_SCHEMA = Schema({
+    'AnalysisType': Or("policy", "rule", "global"),
+},
+                     ignore_extra_keys=True)
+
+GLOBAL_SCHEMA = Schema(
+    {
+        'AnalysisType': Or("global"),
+        'Filename': str,
+        Optional('Description'): str,
+        Optional('Tags'): [str],
+    },
+    ignore_extra_keys=False)
+
+POLICY_SCHEMA = Schema(
     {
         'AnalysisType':
-            Or("policy", "rule"),
+            Or("policy"),
         'Enabled':
             bool,
         'Filename':
@@ -64,8 +78,6 @@ SPEC_SCHEMA = Schema(
             Or("Info", "Low", "Medium", "High", "Critical"),
         Optional('ActionDelaySeconds'):
             int,
-        Optional('AlertFormat'):
-            str,
         Optional('AutoRemediationID'):
             str,
         Optional('AutoRemediationParameters'):
@@ -87,14 +99,51 @@ SPEC_SCHEMA = Schema(
             'Name': str,
             'ResourceType': str,
             'ExpectedResult': bool,
-            'Resource': object
+            'Resource': object,
+        }],
+    },
+    ignore_extra_keys=False)
+
+RULE_SCHEMA = Schema(
+    {
+        'AnalysisType':
+            Or("rule"),
+        'Enabled':
+            bool,
+        'Filename':
+            str,
+        'RuleID':
+            str,
+        'LogTypes': [str],
+        'Severity':
+            Or("Info", "Low", "Medium", "High", "Critical"),
+        Optional('Description'):
+            str,
+        Optional('DedupPeriodMinutes'):
+            int,
+        Optional('DisplayName'):
+            str,
+        Optional('Reference'):
+            str,
+        Optional('Runbook'):
+            str,
+        Optional('Suppressions'): [str],
+        Optional('Tags'): [str],
+        Optional('Reports'): {
+            str: object
+        },
+        Optional('Tests'): [{
+            'Name': str,
+            'LogType': str,
+            'ExpectedResult': bool,
+            'Log': object,
         }],
     },
     ignore_extra_keys=False)
 
 
 def load_module(filename: str) -> Tuple[Any, Any]:
-    """Loads the Policy function module from a file.
+    """Loads the analysis function module from a file.
 
     Args:
         filename: The relative path to the file.
@@ -111,17 +160,21 @@ def load_module(filename: str) -> Tuple[Any, Any]:
     except FileNotFoundError as err:
         print('\t[ERROR] File not found, skipping\n')
         return None, err
+    except Exception as err:  # pylint: disable=broad-except
+        # Catch arbitrary exceptions thrown by user code
+        print('\t[ERROR] Error loading module, skipping\n')
+        return None, err
     return module, None
 
 
-def load_policy_specs(directory: str) -> Iterator[Tuple[str, str, Any]]:
-    """Loads the Policy function module from a file.
+def load_analysis_specs(directory: str) -> Iterator[Tuple[str, str, Any]]:
+    """Loads the analysis specifications from a file.
 
     Args:
-        directory: The relativie path to Panther policies.
+        directory: The relative path to Panther policies or rules.
 
     Yields:
-        A tuple of the relative filepath, directory name, and loaded policy specification dict.
+        A tuple of the relative filepath, directory name, and loaded analysis specification dict.
     """
     for dir_name, _, file_list in os.walk(directory):
         for filename in sorted(file_list):
@@ -148,10 +201,10 @@ def datetime_converted(obj: Any) -> Any:
     return obj
 
 
-def zip_policies(args: argparse.Namespace) -> Tuple[int, str]:
-    """Tests, validates, and then archives all policies into a local zip file.
+def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
+    """Tests, validates, and then archives all policies and rules into a local zip file.
 
-    Returns 1 if the policy test or validation fails.
+    Returns 1 if the analysis tests or validation fails.
 
     Args:
         args: The populated Argparse namespace with parsed command-line arguments.
@@ -159,27 +212,26 @@ def zip_policies(args: argparse.Namespace) -> Tuple[int, str]:
     Returns:
         A tuple of return code and the archive filename.
     """
-    return_code, _ = test_policies(args)
+    return_code, _ = test_analysis(args)
 
     if return_code == 1:
         return return_code, ''
 
-    logging.info('Zipping policies in %s to %s', args.policies,
-                 args.output_path)
+    logging.info('Zipping analysis packs in %s to %s', args.path, args.out)
     # example: 2019-08-05T18-23-25
     # The colon character is not valid in filenames.
     current_time = datetime.now().isoformat(timespec='seconds').replace(
         ':', '-')
-    filename = 'panther-policies'
+    filename = 'panther-analysis'
     return 0, shutil.make_archive(
-        os.path.join(args.output_path, '{}-{}'.format(filename, current_time)),
-        'zip', args.policies)
+        os.path.join(args.out, '{}-{}'.format(filename, current_time)), 'zip',
+        args.path)
 
 
-def upload_policies(args: argparse.Namespace) -> Tuple[int, str]:
-    """Tests, validates, packages, and then uploads all policies into a Panther deployment.
+def upload_analysis(args: argparse.Namespace) -> Tuple[int, str]:
+    """Tests, validates, packages, and uploads all policies and rules into a Panther deployment.
 
-    Returns 1 if the policy tests, validation, or packaging fails.
+    Returns 1 if the analysis tests, validation, or packaging fails.
 
     Args:
         args: The populated Argparse namespace with parsed command-line arguments.
@@ -187,7 +239,7 @@ def upload_policies(args: argparse.Namespace) -> Tuple[int, str]:
     Returns:
         A tuple of return code and the archive filename.
     """
-    return_code, archive = zip_policies(args)
+    return_code, archive = zip_analysis(args)
     if return_code == 1:
         return return_code, ''
 
@@ -204,9 +256,9 @@ def upload_policies(args: argparse.Namespace) -> Tuple[int, str]:
                 json.dumps({
                     'Data': base64.b64encode(zip_bytes).decode('utf-8'),
                     # The UserID is required by Panther for this API call, but we have no way of
-                    # acquiring it and it isn't used for anything. This is a random, valid UUID so
-                    # that the input can be validated by the API.
-                    'UserID': 'c273fd96-88d0-41c4-a74e-941e17832915',
+                    # acquiring it and it isn't used for anything. This is a valid UUID used by the
+                    # Panther deployment tool to indicate this action was performed automatically.
+                    'UserID': '00000000-0000-4000-8000-000000000000',
                 }),
         }
 
@@ -220,6 +272,9 @@ def upload_policies(args: argparse.Namespace) -> Tuple[int, str]:
         response_payload = json.loads(response_str)
 
         if response_payload['statusCode'] != 200:
+            logging.warning(
+                'Failed to upload to Panther\n\tstatus code: %s\n\terror message: %s',
+                response_payload['statusCode'], response_payload['body'])
             return 1, ''
 
         body = json.loads(response_payload['body'])
@@ -230,8 +285,8 @@ def upload_policies(args: argparse.Namespace) -> Tuple[int, str]:
     return 0, ''
 
 
-def test_policies(args: argparse.Namespace) -> Tuple[int, list]:
-    """Imports each Policy/Rule and runs their tests.
+def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
+    """Imports each policy or rule and runs their tests.
 
     Args:
         args: The populated Argparse namespace with parsed command-line arguments.
@@ -239,63 +294,54 @@ def test_policies(args: argparse.Namespace) -> Tuple[int, list]:
     Returns:
         A tuple of the return code, and a list of tuples containing invalid specs and their error.
     """
-    invalid_specs = []
     failed_tests: DefaultDict[str, list] = defaultdict(list)
     tests: List[str] = []
-    logging.info('Testing Policies in %s\n', args.policies)
+    logging.info('Testing analysis packs in %s\n', args.path)
 
-    # First import the globals file
-    specs = list(load_policy_specs(args.policies))
-    for policy_spec_filename, dir_name, policy_spec in specs:
-        if policy_spec.get('PolicyID') != 'aws_globals':
-            continue
+    # First classify each file
+    specs = list(load_analysis_specs(args.path))
+    global_analysis, analysis, invalid_specs = classify_analysis(specs)
+
+    # First import the globals
+    for analysis_spec_filename, dir_name, analysis_spec in global_analysis:
         module, load_err = load_module(
-            os.path.join(dir_name, policy_spec['Filename']))
+            os.path.join(dir_name, analysis_spec['Filename']))
         # If the module could not be loaded, continue to the next
         if load_err:
-            invalid_specs.append((policy_spec_filename, load_err))
+            invalid_specs.append((analysis_spec_filename, load_err))
             break
-        sys.modules['aws_globals'] = module
+        sys.modules['panther'] = module
 
-    # Next import each policy and run its tests
-    for policy_spec_filename, dir_name, policy_spec in specs:
-        if policy_spec.get('PolicyID') == 'aws_globals':
-            continue
+    # Next import each policy or rule and run its tests
+    for analysis_spec_filename, dir_name, analysis_spec in analysis:
+        analysis_id = analysis_spec.get('PolicyID') or analysis_spec['RuleID']
+        print(analysis_id)
 
-        try:
-            SPEC_SCHEMA.validate(policy_spec)
-        except (SchemaError, SchemaMissingKeyError, SchemaForbiddenKeyError,
-                SchemaUnexpectedTypeError) as err:
-            invalid_specs.append((policy_spec_filename, err))
-            continue
-
-        print(policy_spec['PolicyID'])
-
-        # Check if the PolicyID has already been loaded
-        if policy_spec['PolicyID'] in tests:
-            print('\t[ERROR] Conflicting PolicyID\n')
+        # Check if the AnalysisID has already been loaded
+        if analysis_id in tests:
+            print('\t[ERROR] Conflicting AnalysisID\n')
             invalid_specs.append(
-                (policy_spec_filename,
-                 'Conflicting PolicyID: {}'.format(policy_spec['PolicyID'])))
+                (analysis_spec_filename,
+                 'Conflicting AnalysisID: {}'.format(analysis_id)))
             continue
 
         module, load_err = load_module(
-            os.path.join(dir_name, policy_spec['Filename']))
+            os.path.join(dir_name, analysis_spec['Filename']))
         # If the module could not be loaded, continue to the next
         if load_err:
-            invalid_specs.append((policy_spec_filename, load_err))
+            invalid_specs.append((analysis_spec_filename, load_err))
             continue
 
-        tests.append(policy_spec['PolicyID'])
-        if policy_spec['AnalysisType'] == 'policy':
-            run_func = module.policy
-        elif policy_spec['AnalysisType'] == 'rule':
-            run_func = module.rule
-        failed_tests = run_tests(policy_spec, run_func, failed_tests)
+        tests.append(analysis_id)
+        if analysis_spec['AnalysisType'] == 'policy':
+            failed_tests = run_tests(analysis_spec, module.policy, failed_tests)
+        elif analysis_spec['AnalysisType'] == 'rule':
+            failed_tests = run_tests(analysis_spec, module.rule, failed_tests)
         print('')
 
-    for policy_id in failed_tests:
-        print("Failed: {}\n\t{}\n".format(policy_id, failed_tests[policy_id]))
+    for analysis_id in failed_tests:
+        print("Failed: {}\n\t{}\n".format(analysis_id,
+                                          failed_tests[analysis_id]))
 
     for spec_filename, spec_error in invalid_specs:
         print("Invalid: {}\n\t{}\n".format(spec_filename, spec_error))
@@ -303,13 +349,51 @@ def test_policies(args: argparse.Namespace) -> Tuple[int, list]:
     return int(bool(failed_tests or invalid_specs)), invalid_specs
 
 
-def run_tests(policy: Dict[str, Any], run_func: Callable[[TestCase], bool],
+def classify_analysis(
+    specs: List[Tuple[str, str, Any]]
+) -> Tuple[List[Any], List[Any], List[Any]]:
+    # First determine the type of each file
+    global_analysis = []
+    analysis = []
+    invalid_specs = []
+
+    for analysis_spec_filename, dir_name, analysis_spec in specs:
+        try:
+            TYPE_SCHEMA.validate(analysis_spec)
+            if analysis_spec['AnalysisType'] == 'policy':
+                POLICY_SCHEMA.validate(analysis_spec)
+                analysis.append(
+                    (analysis_spec_filename, dir_name, analysis_spec))
+            if analysis_spec['AnalysisType'] == 'rule':
+                RULE_SCHEMA.validate(analysis_spec)
+                analysis.append(
+                    (analysis_spec_filename, dir_name, analysis_spec))
+            if analysis_spec['AnalysisType'] == 'global':
+                GLOBAL_SCHEMA.validate(analysis_spec)
+                global_analysis.append(
+                    (analysis_spec_filename, dir_name, analysis_spec))
+        except (SchemaError, SchemaMissingKeyError, SchemaForbiddenKeyError,
+                SchemaUnexpectedTypeError) as err:
+            invalid_specs.append((analysis_spec_filename, err))
+            continue
+
+    return (global_analysis, analysis, invalid_specs)
+
+
+def run_tests(analysis: Dict[str, Any], run_func: Callable[[TestCase], bool],
               failed_tests: DefaultDict[str, list]) -> DefaultDict[str, list]:
 
-    for unit_test in policy['Tests']:
+    # First check if any tests exist, so we can print a helpful message if not
+    if 'Tests' not in analysis:
+        analysis_id = analysis.get('PolicyID') or analysis['RuleID']
+        print('\tNo tests configured for {}'.format(analysis_id))
+        return failed_tests
+
+    for unit_test in analysis['Tests']:
         try:
-            test_case = TestCase(unit_test['Resource'],
-                                 unit_test['ResourceType'])
+            test_case = TestCase(
+                unit_test.get('Resource') or unit_test['Log'],
+                unit_test.get('ResourceType') or unit_test['LogType'])
             result = run_func(test_case)
         except KeyError as err:
             print("KeyError: {0}".format(err))
@@ -317,7 +401,8 @@ def run_tests(policy: Dict[str, Any], run_func: Callable[[TestCase], bool],
         test_result = 'PASS'
         if result != unit_test['ExpectedResult']:
             test_result = 'FAIL'
-            failed_tests[policy['PolicyID']].append(unit_test['Name'])
+            failed_tests[analysis.get('PolicyID') or
+                         analysis['RuleID']].append(unit_test['Name'])
         print('\t[{}] {}'.format(test_result, unit_test['Name']))
 
     return failed_tests
@@ -326,47 +411,56 @@ def run_tests(policy: Dict[str, Any], run_func: Callable[[TestCase], bool],
 def setup_parser() -> argparse.ArgumentParser:
     parser = argparse.ArgumentParser(
         description=
-        'Panther Analaysis Tool: A tool for writing, testing, and packaging Panther Policies/Rules',
+        'Panther Analysis Tool: A command line tool for managing Panther policies and rules.',
         prog='panther_analysis_tool')
     parser.add_argument('--version',
                         action='version',
-                        version='panther_analysis_tool 0.1.4')
+                        version='panther_analysis_tool 0.2.1')
     subparsers = parser.add_subparsers()
 
     test_parser = subparsers.add_parser(
-        'test', help='Validate policy specifications and run policy tests.')
-    test_parser.add_argument('--policies',
-                             type=str,
-                             help='The relative path to Panther policies.',
-                             required=True)
-    test_parser.set_defaults(func=test_policies)
+        'test',
+        help='Validate analysis specifications and run policy and rule tests.')
+    test_parser.add_argument(
+        '--path',
+        type=str,
+        help='The relative path to Panther policies and rules.',
+        required=True)
+    test_parser.set_defaults(func=test_analysis)
 
     zip_parser = subparsers.add_parser(
         'zip',
-        help='Create an archive of local policies for uploading to Panther.')
-    zip_parser.add_argument('--policies',
-                            type=str,
-                            help='The relative path to Panther policies.',
-                            required=True)
-    zip_parser.add_argument('--output-path',
-                            type=str,
-                            help='The path to write zipped policies to.',
-                            required=True)
-    zip_parser.set_defaults(func=zip_policies)
+        help=
+        'Create an archive of local policies and rules for uploading to Panther.'
+    )
+    zip_parser.add_argument(
+        '--path',
+        type=str,
+        help='The relative path to Panther policies and rules.',
+        required=True)
+    zip_parser.add_argument(
+        '--out',
+        type=str,
+        help='The path to write zipped policies and rules to.',
+        required=True)
+    zip_parser.set_defaults(func=zip_analysis)
 
     upload_parser = subparsers.add_parser(
-        'upload', help='Upload specified policies to a Panther deployment.')
-    upload_parser.add_argument('--policies',
-                               type=str,
-                               help='The relative path to Panther policies.',
-                               required=True)
+        'upload',
+        help='Upload specified policies and rules to a Panther deployment.')
     upload_parser.add_argument(
-        '--output-path',
+        '--path',
+        type=str,
+        help='The relative path to Panther policies and rules.',
+        required=True)
+    upload_parser.add_argument(
+        '--out',
         default='.',
         type=str,
-        help='The location to store a local copy of the packaged policies.',
+        help=
+        'The location to store a local copy of the packaged policies and rules.',
         required=False)
-    upload_parser.set_defaults(func=upload_policies)
+    upload_parser.set_defaults(func=upload_analysis)
 
     return parser
 
